//*******************************************************************************
// attention 
// Copyright (C) 2018 MACNICA,Inc. All Rights Reserved.
//   This software is licensed "AS IS". 
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

`default_nettype none

module isp_white_balance 
#(
    parameter DATA_WIDTH    = 32,   // Data bit width
    parameter BITS_SIMBOL   = 8,    // Bits per symbol
    parameter SIMBOL_PARA   = 3     // Simbols in parallel
)
(
    input  wire                                 clock,
    input  wire                                 reset_n,

    // Avalon-ST Sink
    input  wire                                 in_valid,
    input  wire                                 in_sop,
    input  wire                                 in_eop,
    input  wire [BITS_SIMBOL*SIMBOL_PARA-1:0]   in_data,
    input  wire [BITS_SIMBOL*SIMBOL_PARA-1:0]   in_yuv,
    // Avalon-ST Source
    output wire [BITS_SIMBOL-1:0]               out_u,
    output wire [BITS_SIMBOL-1:0]               out_v,
    
    input  wire [DATA_WIDTH-1:0]                coef_y
);
    localparam TA_IDLE = 2'd0;
    localparam TA_CTRL = 2'd1;
    localparam TA_DATA = 2'd2;
    localparam BITS_MSB = BITS_SIMBOL-1;
    localparam BITS_HALF = {{(BITS_MSB){1'b0}}, 1'b1} << (BITS_MSB);

    reg     [1:0]   st_avst;
    reg     [31:0]  add_u;
    reg     [31:0]  add_v;
    reg     [21:0]  count;
    reg     [21:0]  total;
    reg     [31:0]  sum_u;
    reg     [31:0]  sum_v;
    reg     [31:0]  sum_uv;
    reg     [31:0]  coef_u;
    reg     [31:0]  coef_v;
    reg             avg_act;
    reg     [3:0]   avg_pipe;
    
    wire    [BITS_SIMBOL-1:0]   in_data_y;
    wire    [BITS_SIMBOL-1:0]   in_data_v;
    wire    [BITS_SIMBOL-1:0]   in_data_u;

    assign  in_data_u = in_yuv[BITS_SIMBOL*1-1 : BITS_SIMBOL*0];
    assign  in_data_v = in_yuv[BITS_SIMBOL*2-1 : BITS_SIMBOL*1];
    assign  in_data_y = in_yuv[BITS_SIMBOL*3-1 : BITS_SIMBOL*2];

    always @(posedge clock or negedge reset_n) begin
        if (!reset_n) begin
            st_avst     <= TA_IDLE;
            count       <= 22'd1;
            add_u       <= BITS_HALF;
            add_v       <= BITS_HALF;
        end else begin
            if(in_valid) begin
                case(st_avst)
                    TA_IDLE: begin
                        if(in_sop) begin
                            if(in_data[3:0] == 4'hF) begin
                                st_avst <= TA_CTRL;
                            end else
                            if(in_data[3:0] == 4'h0) begin
                                st_avst <= TA_DATA;
                                add_u   <= 0;
                                add_v   <= 0;
                                count   <= 22'd0;
                            end
                        end
                    end
                    TA_CTRL: begin
                        if(in_eop) begin
                            st_avst     <= TA_IDLE;
                        end
                    end
                    TA_DATA: begin
                        add_u <= add_u + {{(32-BITS_SIMBOL){1'b0}}, in_data_u};
                        add_v <= add_v + {{(32-BITS_SIMBOL){1'b0}}, in_data_v};
                        count <= count + 22'd1;
                        if(in_eop) begin
                            st_avst <= TA_IDLE;
                        end
                    end
                endcase
            end
        end
    end

    // int avg_u = sum_u / total;
    wire    [31:0]  avg_uv;
    lpm_divide  int_div_uv (
        .clock ( clock ),   
        .denom ( total ),       // 22
        .numer ( sum_uv ),       // 32
        .quotient ( avg_uv ),    // 32
        .remain (),
        .aclr (!reset_n),
        .clken (1'b1)
    );
    defparam
        int_div_uv.lpm_drepresentation = "UNSIGNED",
        int_div_uv.lpm_hint = "LPM_REMAINDERPOSITIVE=TRUE",
        int_div_uv.lpm_nrepresentation = "UNSIGNED",
        int_div_uv.lpm_pipeline = 10,
        int_div_uv.lpm_type = "LPM_DIVIDE",
        int_div_uv.lpm_widthd = 22,
        int_div_uv.lpm_widthn = 32;
        
    // int aof_u = avg_u - 128;
    wire    [BITS_SIMBOL:0]   aof_uv;
    lpm_add_sub int_sub_uv (
        .clock ( clock ),
        .dataa ( {1'b0, avg_uv[BITS_MSB:0]} ),  //  9
        .datab ( {1'b0, BITS_HALF} ),
        .result ( aof_uv )
    );
    defparam
        int_sub_uv.lpm_direction = "SUB",
        int_sub_uv.lpm_hint = "ONE_INPUT_IS_CONSTANT=YES,CIN_USED=NO",
        int_sub_uv.lpm_pipeline = 1,
        int_sub_uv.lpm_representation = "SIGNED",
        int_sub_uv.lpm_type = "LPM_ADD_SUB",
        int_sub_uv.lpm_width = BITS_SIMBOL+1;

    // float coef_u = aof_u;
    wire    [31:0]  coef_uv;
    int2fp  int2fp_auv (
        .clk    ( clock ),
        .areset (!reset_n),
        .a      ( {{BITS_MSB{aof_uv[BITS_MSB]}}, aof_uv} ),
        .q      ( coef_uv )
    );

    //localparam AVG_PIPE = int_div_uv.lpm_pipeline + int_sub_uv.lpm_pipeline + 3;
    localparam AVG_PIPE = 14;

    // coef_u,v = float((sum_u,v / total) - 128);
    always @(posedge clock or negedge reset_n) begin
        if (!reset_n) begin
            total    <= 22'd1;
            sum_u    <= BITS_HALF;
            sum_v    <= BITS_HALF;
            avg_act  <= 1'b0;
            avg_pipe <= 4'd0;
            sum_uv   <= 0;
            coef_u   <= 0;
            coef_v   <= 0;
        end else begin
            if(in_valid & in_sop & (st_avst == TA_IDLE) & (in_data[3:0] == 4'hF)) begin
                sum_u    <= add_u;
                sum_v    <= add_v;
                total    <= count;
                avg_act  <= 1'b1;
                avg_pipe <= 4'd0;
            end else
            if(avg_act) begin
                case(avg_pipe )
                    0: begin
                        sum_uv <= sum_u;
                    end
                    1: begin
                        sum_uv <= sum_v;
                    end
                    AVG_PIPE: begin
                        coef_u <= coef_uv;
                    end
                    AVG_PIPE + 1: begin
                        coef_v <= coef_uv;
                        avg_act <= 1'b0;
                    end
                endcase
                avg_pipe <= avg_pipe + 4'd1;
            end
        end
    end

    //====== 2 clk ======//
    // float fpu_y = float(ptr[x * 3 + 0]);
    wire    [31:0]  fpu_y;
    int2fp  int2fp_y (
        .clk    ( clock ),
        .areset (!reset_n),
        .a      ( {{(16-BITS_SIMBOL){1'b0}}, in_data_y} ),
        .q      ( fpu_y )
    );

    //====== 3 clk ======//
    // float cal_y = fpu_y * coef_y;
    wire    [31:0]  cal_y;
    fp_mul  fp_mul_y (
        .clk    ( clock ),
        .areset (!reset_n),
        .a      ( fpu_y ),
        .b      ( coef_y ),
        .q      ( cal_y )
    );
        
    //====== 3 clk ======//
    // float cal_u = coef_u * cal_y;
    wire    [31:0]  cal_u;
    fp_mul  fp_mul_u (
        .clk    ( clock ),
        .areset (!reset_n),
        .a      ( coef_u ),
        .b      ( cal_y ),
        .q      ( cal_u )
    );
    // float cal_v = coef_v * cal_y;
    wire    [31:0]  cal_v;
    fp_mul  fp_mul_v (
        .clk    ( clock ),
        .areset (!reset_n),
        .a      ( coef_v ),
        .b      ( cal_y ),
        .q      ( cal_v )
    );

    //====== 1 clk ======//
    // short int_u = (float)cal_u;
    wire    [15:0]  int_u;
    fp2int  fp2int_u (
        .clk    ( clock ),
        .areset (!reset_n),
        .a      ( cal_u ),
        .q      ( int_u )
    );
    // short int_v = (float)cal_v;
    wire    [15:0]  int_v;
    fp2int  fp2int_v (
        .clk    ( clock ),
        .areset (!reset_n),
        .a      ( cal_v ),
        .q      ( int_v )
    );

    // UV data delay 9 clock
    wire    [BITS_SIMBOL*2-1:0] vu_delay;
    wire    [BITS_SIMBOL-1:0]   dly_in_u;
    wire    [BITS_SIMBOL-1:0]   dly_in_v;
    isp_delay
    #(
        .DATA_WIDTH  (BITS_SIMBOL * 2), 
        .DELAY (9)
    )   u_uv_delay
    (
        .clock    ( clock ),
        .reset_n  ( reset_n ),
        .in_data  ( {in_data_u, in_data_v}),
        .out_data ( vu_delay )
    );

    assign  dly_in_v = vu_delay[BITS_SIMBOL*1-1 : BITS_SIMBOL*0];
    assign  dly_in_u = vu_delay[BITS_SIMBOL*2-1 : BITS_SIMBOL*1];

    // short res_u = dly_in_u - int_u;
    wire    [BITS_SIMBOL+1:0]   res_u;
    lpm_add_sub res_sub_u (
        .clock  ( clock ),
        .dataa  ( {2'b0, dly_in_u} ),   //  9
        .datab  ( int_u[BITS_SIMBOL+1:0] ),
        .result ( res_u )
    );
    defparam
        res_sub_u.lpm_direction = "SUB",
        res_sub_u.lpm_hint = "ONE_INPUT_IS_CONSTANT=YES,CIN_USED=NO",
        res_sub_u.lpm_pipeline = 3,
        res_sub_u.lpm_representation = "SIGNED",
        res_sub_u.lpm_type = "LPM_ADD_SUB",
        res_sub_u.lpm_width = BITS_SIMBOL+2;

    assign  out_u = (res_u[BITS_SIMBOL+1:BITS_SIMBOL+1])? {(BITS_SIMBOL){1'b0}} : (res_u[BITS_SIMBOL:BITS_SIMBOL])? {(BITS_SIMBOL){1'b1}} : res_u[BITS_SIMBOL-1:0];

    // short res_v = dly_in_v - int_v;
    wire    [BITS_SIMBOL+1:0]   res_v;
    lpm_add_sub res_sub_v (
        .clock  ( clock ),
        .dataa  ( {2'b0, dly_in_v} ),   //  9
        .datab  ( int_v[BITS_SIMBOL+1:0] ),
        .result ( res_v )
    );
    defparam
        res_sub_v.lpm_direction = "SUB",
        res_sub_v.lpm_hint = "ONE_INPUT_IS_CONSTANT=YES,CIN_USED=NO",
        res_sub_v.lpm_pipeline = 3,
        res_sub_v.lpm_representation = "SIGNED",
        res_sub_v.lpm_type = "LPM_ADD_SUB",
        res_sub_v.lpm_width = BITS_SIMBOL+2;

    assign  out_v = (res_v[BITS_SIMBOL+1:BITS_SIMBOL+1])? {(BITS_SIMBOL){1'b0}} : (res_v[BITS_SIMBOL:BITS_SIMBOL])? {(BITS_SIMBOL){1'b1}} : res_v[BITS_SIMBOL-1:0];

endmodule

`default_nettype wire