//*******************************************************************************
// attention 
// Copyright (C) 2018 MACNICA,Inc. All Rights Reserved.
//   This software is licensed "AS IS". 
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

`default_nettype none

// R = 1.164(Y-16)                + 1.596(V-128)
// G = 1.164(Y-16) - 0.392(U-128) - 0.813(V-128) 
// B = 1.164(Y-16) + 2.017(U-128)

module isp_yuv2rgb
#(
    parameter BITS_SIMBOL = 8,  // Data bit width
    parameter SIMBOL_PARA = 3   // Simbols in parallel
)
(
    input  wire                                 clock,
    input  wire                                 reset_n,

    input  wire [BITS_SIMBOL*SIMBOL_PARA-1:0]   in_data,
    output wire [BITS_SIMBOL*SIMBOL_PARA-1:0]   out_data
);
    localparam CALC_BITS = 18;
    localparam COEF_R_Y  = 18'h094FE;   // 1.164 * 2^15 = 38142 = ‭94FE‬
    localparam COEF_R_U  = 18'h00000;   // 0
    localparam COEF_R_V  = 18'h0CC4A;   // 1.596 * 2^15 = 52298 = ‭CC4A‬
    localparam COEF_G_Y  = 18'h094FE;   //  1.164 * 2^15 =  38142 = ‭94FE‬
    localparam COEF_G_U  = 18'hFCDD3;   // -0.392 * 2^15 = -12845 = ‭FFFFFFFFFFFFCDD3‬
    localparam COEF_G_V  = 18'hF97F0;   // -0.813 * 2^15 = -26640 = ‭FFFFFFFFFFFF97F0‬
    localparam COEF_B_Y  = 18'h094FE;   // 1.164 * 2^15 = 38142 = ‭94FE‬
    localparam COEF_B_U  = 18'h1022D;   // 2.017 * 2^15 = 66093 = ‭1022D‬
    localparam COEF_B_V  = 18'h00000;   // 0

    wire [CALC_BITS-1:0]    in_y;
    wire [CALC_BITS-1:0]    in_u;
    wire [CALC_BITS-1:0]    in_v;

    assign in_y = {28'b0, in_data[BITS_SIMBOL*3-1 : BITS_SIMBOL*2]};
    assign in_v = {28'b0, in_data[BITS_SIMBOL*2-1 : BITS_SIMBOL*1]};
    assign in_u = {28'b0, in_data[BITS_SIMBOL*1-1 : BITS_SIMBOL*0]};

    reg signed [CALC_BITS-1:0]      reg_y;
    reg signed [CALC_BITS-1:0]      reg_u;
    reg signed [CALC_BITS-1:0]      reg_v;

    always @(posedge clock or negedge reset_n) begin
        if (!reset_n) begin
            reg_y <= {(CALC_BITS){1'b0}};
            reg_u <= {(CALC_BITS){1'b0}};
            reg_v <= {(CALC_BITS){1'b0}};
        end else begin
            reg_y <= in_y - 9'd16;
            reg_u <= in_u - 9'd128;
            reg_v <= in_v - 9'd128;
        end
    end

    wire [CALC_BITS*2-1:0]  res_r;
    wire [CALC_BITS*2-1:0]  res_g;
    wire [CALC_BITS*2-1:0]  res_b;

    // R = 1.164(Y-16) + 0.000(U-128) + 1.596(V-128)
    isp_csc_muladd 
    #(
        .CALC_BITS (CALC_BITS)
    )   u_csc_r
    (
        .clock      ( clock ),
        .reset_n    ( reset_n ),
        .coef_0     ( COEF_R_Y ),  // 1.164 * 2^15 = 38142 = ‭94FE‬
        .coef_1     ( COEF_R_U ),  // 0
        .coef_2     ( COEF_R_V ),  // 1.596 * 2^15 = 52298 = ‭CC4A‬
        .in_0       ( reg_y ),
        .in_1       ( reg_u ),
        .in_2       ( reg_v ),
        
        .out        ( res_r )
    );
    // G = 1.164(Y-16) - 0.392(U-128) - 0.813(V-128) 
    isp_csc_muladd 
    #(
        .CALC_BITS (CALC_BITS)
    )   u_csc_g
    (
        .clock      ( clock ),
        .reset_n    ( reset_n ),
        .coef_0     ( COEF_G_Y ),  //  1.164 * 2^15 =  38142 = ‭94FE‬
        .coef_1     ( COEF_G_U ),  // -0.392 * 2^15 = -12845 = ‭FFFFFFFFFFFFCDD3‬
        .coef_2     ( COEF_G_V ),  // -0.813 * 2^15 = -26640 = ‭FFFFFFFFFFFF97F0‬
        .in_0       ( reg_y ),
        .in_1       ( reg_u ),
        .in_2       ( reg_v ),
        
        .out        ( res_g )
    );

    // B = 1.164(Y-16) + 2.017(U-128) + 0.000(V-128)
    isp_csc_muladd 
    #(
        .CALC_BITS (CALC_BITS)
    )   u_csc_b
    (
        .clock      ( clock ),
        .reset_n    ( reset_n ),
        .coef_0     ( COEF_B_Y ),  // 1.164 * 2^15 = 38142 = ‭94FE‬
        .coef_1     ( COEF_B_U ),  // 2.017 * 2^15 = 66093 = ‭1022D‬
        .coef_2     ( COEF_B_V ),  // 0
        .in_0       ( reg_y ),
        .in_1       ( reg_u ),
        .in_2       ( reg_v ),
        
        .out        ( res_b )
    );
    
    wire [BITS_SIMBOL-1:0]  out_r;
    wire [BITS_SIMBOL-1:0]  out_g;
    wire [BITS_SIMBOL-1:0]  out_b;

    assign out_r = res_r[CALC_BITS*2-1]? {(BITS_SIMBOL){1'b0}} : 
                   (res_r[CALC_BITS*2-2:23] & {(12){1'b1}})? {(BITS_SIMBOL){1'b1}} : 
                   res_r[22:15];
    
    assign out_g = res_g[CALC_BITS*2-1]? {(BITS_SIMBOL){1'b0}} : 
                   (res_g[CALC_BITS*2-2:23] & {(12){1'b1}})? {(BITS_SIMBOL){1'b1}} : 
                   res_g[22:15];
                   
    assign out_b = res_b[CALC_BITS*2-1]? {(BITS_SIMBOL){1'b0}} : 
                   (res_b[CALC_BITS*2-2:23] & {(12){1'b1}})? {(BITS_SIMBOL){1'b1}} : 
                   res_b[22:15];

    assign out_data = {out_r, out_g, out_b};
    
endmodule

`default_nettype wire