//*******************************************************************************
// attention 
// Copyright (C) 2018 MACNICA,Inc. All Rights Reserved.
//   This software is licensed "AS IS". 
//   Please perform use of this software by a user's own responsibility and expense.
//   It cannot guarantee in the maker side about the damage which occurred by the ab-
//   ility not to use or use this software, and all damage that occurred secondarily.
//*******************************************************************************

// synthesis translate_off
`timescale 1ns / 1ps
// synthesis translate_on

`default_nettype none

module tiny_isp 
#(
    parameter ADRS_WIDTH    = 3,    // Address bit width
    parameter DATA_WIDTH    = 32,   // Data bit width
    parameter BITS_SIMBOL   = 8,    // Bits per symbol
    parameter SIMBOL_PARA   = 3,    // Simbols in parallel
    parameter DEF_COEF_Y    = 32'h3B8D5A27, // Y Coefficient : 1/255*1.1
    parameter BIAS_R        = -8'd40,   // -40
    parameter BIAS_G        = -8'd40,   // -40
    parameter BIAS_B        = -8'd40,   // -40
    parameter GAIN_R        = 8'd68,    // 1.07 * 2^6 = 68
    parameter GAIN_G        = 8'd57,    // 0.90 * 2^6 = 57
    parameter GAIN_B        = 8'd67,    // 1.05 * 2^6 = 67 
    parameter MEM_FILE      = "gamma.mif"   // Gamma table file name
)
(
    input   wire                                    clock,
    input   wire                                    reset_n,

    // Avalon-MM Slave
    input   wire    [ADRS_WIDTH-1:0]                address,
    input   wire                                    write,
    input   wire    [DATA_WIDTH-1:0]                writedata,
    input   wire                                    read,
    output  wire    [DATA_WIDTH-1:0]                readdata,

    // Avalon-ST Sink
    output  wire                                    din_ready,
    input   wire                                    din_valid,
    input   wire                                    din_sop,
    input   wire                                    din_eop,
    input   wire    [BITS_SIMBOL*SIMBOL_PARA-1:0]   din_data,
    // Avalon-ST Source
    input   wire                                    dout_ready,
    output  wire                                    dout_valid,
    output  wire                                    dout_sop,
    output  wire                                    dout_eop,
    output  wire    [BITS_SIMBOL*SIMBOL_PARA-1:0]   dout_data
);
    wire [DATA_WIDTH-1:0]               coef_y;
    wire [DATA_WIDTH-1:0]               bias_r;
    wire [DATA_WIDTH-1:0]               bias_g;
    wire [DATA_WIDTH-1:0]               bias_b;
    wire [DATA_WIDTH-1:0]               gain_r;
    wire [DATA_WIDTH-1:0]               gain_g;
    wire [DATA_WIDTH-1:0]               gain_b;
    wire                                dly_packet;
    wire                                dly_valid;
    wire                                dly_sop;
    wire                                dly_eop;
    wire [BITS_SIMBOL*SIMBOL_PARA-1:0]  dly_data;
    wire [BITS_SIMBOL-1:0]              out_u;
    wire [BITS_SIMBOL-1:0]              out_v;
    wire [BITS_SIMBOL-1:0]              out_y;
    wire [BITS_SIMBOL*SIMBOL_PARA-1:0]  gan_data;
    wire [BITS_SIMBOL*SIMBOL_PARA-1:0]  yuv_data;
    wire [BITS_SIMBOL*SIMBOL_PARA-1:0]  rgb_data;
    
    isp_input
    #(
        .BITS_SIMBOL (BITS_SIMBOL),
        .SIMBOL_PARA (SIMBOL_PARA),
        .DELAY       (26)
    )   u_isp_input
    (
        .clock                  ( clock ),
        .reset_n                ( reset_n ),
        // Avalon-ST Sink
        .in_valid               ( din_valid ),
        .in_sop                 ( din_sop ),
        .in_eop                 ( din_eop ),
        .in_data                ( din_data ),

        .out_packet             ( dly_packet ),
        .out_valid              ( dly_valid ),
        .out_sop                ( dly_sop ),
        .out_eop                ( dly_eop ),
        .out_data               ( dly_data )
    );

    // Color Balance　[Latency:3]
    isp_color_balance
    #(
        .BITS_SIMBOL (BITS_SIMBOL),
        .SIMBOL_PARA (SIMBOL_PARA)
    )   u_isp_color_balance
    (
        .clock                  ( clock ),
        .reset_n                ( reset_n ),

        .bias_r                 ( bias_r[BITS_SIMBOL-1:0] ),
        .bias_g                 ( bias_g[BITS_SIMBOL-1:0] ),
        .bias_b                 ( bias_b[BITS_SIMBOL-1:0] ),

        .gain_r                 ( gain_r[BITS_SIMBOL-1:0] ),
        .gain_g                 ( gain_g[BITS_SIMBOL-1:0] ),
        .gain_b                 ( gain_b[BITS_SIMBOL-1:0] ),

        .in_data                ( din_data ),
        .out_data               ( gan_data )
    );
    
    // RGB to YUV Convert [Latency:5]
    isp_rgb2yuv
    #(
        .BITS_SIMBOL (BITS_SIMBOL),
        .SIMBOL_PARA (SIMBOL_PARA)
    )   u_isp_rgb2yuv
    (
        .clock                  ( clock ),
        .reset_n                ( reset_n ),
        .in_data                ( gan_data ),
        .out_data               ( yuv_data )
    );
    
    // Luminance gamma correction[Latency:12]
    isp_gamma_corrector
    #(
        .BITS_SIMBOL (BITS_SIMBOL),
        .DELAY       (12),
        .MEM_FILE    (MEM_FILE)
    )   u_isp_gamma_corrector
    (
        .clock                  ( clock ),
        .reset_n                ( reset_n ),
        // Avalon-ST Sink
        .in_data                ( yuv_data[BITS_SIMBOL*3-1 : BITS_SIMBOL*2] ),
        .out_data               ( out_y )
    );
    
    // Auto white balance [Latency:12]
    isp_white_balance
    #(
        .DATA_WIDTH  (DATA_WIDTH), 
        .BITS_SIMBOL (BITS_SIMBOL),
        .SIMBOL_PARA (SIMBOL_PARA)
    )   u_isp_white_balance
    (
        .clock                  ( clock ),
        .reset_n                ( reset_n ),
        // Avalon-ST Sink
        .in_valid               ( din_valid ),
        .in_sop                 ( din_sop ),
        .in_eop                 ( din_eop ),
        .in_data                ( din_data ),
        .in_yuv                 ( yuv_data ),

        .out_u                  ( out_u ),
        .out_v                  ( out_v ),
        // Input
        .coef_y                 ( coef_y )
    );  

    // YUV to RGB Convert [Latency:5]
    isp_yuv2rgb
    #(
        .BITS_SIMBOL (BITS_SIMBOL),
        .SIMBOL_PARA (SIMBOL_PARA)
    )   u_isp_yuv2rgb
    (
        .clock                  ( clock ),
        .reset_n                ( reset_n ),
        .in_data                ( {out_y, out_v, out_u} ),
        .out_data               ( rgb_data )
    );

    isp_output
    #(
        .LATENCY    (26),
        .SRC_WIDTH  (BITS_SIMBOL * SIMBOL_PARA), 
        .FIFO_DEPTH (5)
    )   u_isp_output
    (
        .clock                  ( clock ),
        .reset_n                ( reset_n ),
        // Avalon-ST Sink
        .in_ready               ( din_ready ),
        .in_valid               ( dly_valid ),
        .in_sop                 ( dly_sop ),
        .in_eop                 ( dly_eop ),
        .in_data                ( (dly_packet)? rgb_data : dly_data ),
        // Avalon-ST Source
        .out_ready              ( dout_ready ),
        .out_valid              ( dout_valid ),
        .out_sop                ( dout_sop ),
        .out_eop                ( dout_eop ),
        .out_data               ( dout_data )
    );  

    isp_avmm_slave
    #(
        .DATA_WIDTH (DATA_WIDTH),
        .ADRS_WIDTH (ADRS_WIDTH),
        .DEF_COEF_Y (DEF_COEF_Y),
        .BIAS_R     (BIAS_R),
        .BIAS_G     (BIAS_G),
        .BIAS_B     (BIAS_B),
        .GAIN_R     (GAIN_R),
        .GAIN_G     (GAIN_G),
        .GAIN_B     (GAIN_B)
    )   u_isp_avmm_slave
    (
        .clock                  ( clock ),
        .reset_n                ( reset_n ),
        // Output
        .coef_y                 ( coef_y ),
        .bias_r                 ( bias_r ),
        .bias_g                 ( bias_g ),
        .bias_b                 ( bias_b ),
        .gain_r                 ( gain_r ),
        .gain_g                 ( gain_g ),
        .gain_b                 ( gain_b ),
        // Avalon-MM Slave
        .address                ( address ),
        .write                  ( write ),
        .writedata              ( writedata ),
        .read                   ( read ),
        .readdata               ( readdata )
    );

endmodule

`default_nettype wire