#!/usr/bin/env python3

import fcntl
import os
from time       import sleep
#import cv2
import numpy        as np
import mmap
import math
from smbus2     import SMBus, i2c_msg
import argparse

MESSAGE_ON              = False
DATA_MODE               = "RAW10"
LW_BRIDGE_SIZE          = 0x1000

SCL_OFFSET              = 0x200
ALTSCL_CONTROL          = 0
ALTSCL_WIDTH            = 3
ALTSCL_HEIGHT           = 4

PIO_OFFSET              = 0x40
ALTPIO_DATA             = 0

I2C_SADR_OV5647         = 0x36
I2C_SADR_TC358748       = 0x0e

dtype           = [
    ('adrs', 'uint16'),
    ('data', 'uint8')
]


##
## Register settings from driver for OmniVision OV5647 cameras
##   https://github.com/torvalds/linux/blob/master/drivers/media/i2c/ov5647.c
##

mode_1920x1080  = np.array( [

    (0x0100, 0x00),
    (0x0103, 0x01),
    (0x3034, 0x1a),
    (0x3035, 0x21),
    (0x3036, 0x62),
    (0x303c, 0x11),
    (0x3106, 0xf5),
    (0x3821, 0x06),
    (0x3820, 0x00),
    (0x3827, 0xec),
    (0x370c, 0x03),
    (0x3612, 0x5b),
    (0x3618, 0x04),
    (0x5000, 0x06),
    (0x5002, 0x41),
    (0x5003, 0x08),
    (0x5a00, 0x08),
    (0x3000, 0x00),
    (0x3001, 0x00),
    (0x3002, 0x00),
    (0x3016, 0x08),
    (0x3017, 0xe0),
    (0x3018, 0x44),
    (0x301c, 0xf8),
    (0x301d, 0xf0),
#    (0x3a18, 0x00),
#    (0x3a19, 0xf8),
    (0x3c01, 0x80),
    (0x3b07, 0x0c),
    (0x380c, 0x09),
    (0x380d, 0x70),
    (0x3814, 0x11),
    (0x3815, 0x11),
    (0x3708, 0x64),
    (0x3709, 0x12),
#    (0x3808, 0x07),
#    (0x3809, 0x80),
#    (0x380a, 0x04),
#    (0x380b, 0x38),
    (0x3800, 0x01),
    (0x3801, 0x5c),
    (0x3802, 0x01),
    (0x3803, 0xb2),
    (0x3804, 0x08),
    (0x3805, 0xe3),
    (0x3806, 0x05),
    (0x3807, 0xf1),
    (0x3811, 0x04),
    (0x3813, 0x02),
    (0x3630, 0x2e),
    (0x3632, 0xe2),
    (0x3633, 0x23),
    (0x3634, 0x44),
    (0x3636, 0x06),
    (0x3620, 0x64),
    (0x3621, 0xe0),
    (0x3600, 0x37),
    (0x3704, 0xa0),
    (0x3703, 0x5a),
    (0x3715, 0x78),
    (0x3717, 0x01),
    (0x3731, 0x02),
    (0x370b, 0x60),
    (0x3705, 0x1a),
    (0x3f05, 0x02),
    (0x3f06, 0x10),
    (0x3f01, 0x0a),
    (0x3a08, 0x01),
    (0x3a09, 0x4b),
    (0x3a0a, 0x01),
    (0x3a0b, 0x13),
    (0x3a0d, 0x04),
    (0x3a0e, 0x03),
#    (0x3a0f, 0x58),
#    (0x3a10, 0x50),
#    (0x3a1b, 0x58),
#    (0x3a1e, 0x50),
#    (0x3a11, 0x60),
#    (0x3a1f, 0x28),
    (0x4001, 0x02),
    (0x4004, 0x04),
    (0x4000, 0x09),
    (0x4837, 0x19),
    (0x4800, 0x34),
#    (0x3503, 0x03),
    (0x5180, 0x40),

    (0x3503, 0x00               ),  # [1] AGC manual, [0] AEC manual (def.0x00)
    (0x3a11, 0x40				),	# [7:0] vpt_high (def.0xd0)
    (0x3a1f, 0x08				),	# [7:0] vpt_low  (def.0x40)
    (0x3a0f, 0x20				),	# [7:0] wpt  (def.0x78)
    (0x3a10, 0x18				),	# [7:0] bpt  (def.0x68)
    (0x3a1b, 0x20				),	# [7:0] wpt2 (def.0x78)
    (0x3a1e, 0x18				),	# [7:0] bpt2 (def.0x68)
    (0x3a18, 0x00               ),  # [1:0] gain_ceiling[9:8] (def.0x00)
    (0x3a19, 0x80               ),  # [7:0] gain_ceiling[7:0] (def.0x7c)
    (0x5001, 0x01               ),  # [0] awb_en (def.0x01)

    (0x3808, 1920       >> 8    ),  # [3:0] DVP output horizontal width[11:8]
    (0x3809, 1920       &  0xff ),	# [7:0] DVP output horizontal width[ 7:0]
    (0x380a, 1080       >> 8    ),	# [3:0] DVP output vertical height[11:8]
    (0x380b, 1080       &  0xff ),	# [7:0] DVP output vertical height[ 7:0]
    (0x380e, 1120       >> 8    ),  # [3:0] Total vertical size[9:8]
    (0x380f, 1120       &  0xff ),	# [7:0] Total vertical size[7:0]

#	(0x3820, 0x46               ),  # [2] r_vflip_isp,  [1] r_vflip_snr,  [0] Vertical binning   (def.0x40)
#	(0x3821, 0x00               ),  # [2] r_mirror_isp, [1] r_mirror_snr, [0] Horizontal binning (def.0x00)

    (0x0100, 0x01),

], dtype=dtype )


mode_1280x960  = np.array( [

    (0x0100, 0x00),
    (0x0103, 0x01),
    (0x3034, 0x1a),
    (0x3035, 0x21),
    (0x3036, 0x62),
    (0x303c, 0x11),
    (0x3106, 0xf5),
    (0x3821, 0x06),

    (0x3820, 0x00),
    (0x3827, 0xec),
    (0x370c, 0x03),
    (0x3612, 0x5b),
    (0x3618, 0x04),
    (0x5000, 0x06),
    (0x5002, 0x41),
    (0x5003, 0x08),
    (0x5a00, 0x08),
    (0x3000, 0x00),
    (0x3001, 0x00),

    (0x3002, 0x00),
    (0x3016, 0x08),
    (0x3017, 0xe0),
    (0x3018, 0x44),
    (0x301c, 0xf8),
    (0x301d, 0xf0),
#    (0x3a18, 0x00),
#    (0x3a19, 0xf8),
    (0x3c01, 0x80),
    (0x3b07, 0x0c),
    (0x380c, 0x09),
    (0x380d, 0x70),
    (0x3814, 0x11),

    (0x3815, 0x11),
    (0x3708, 0x64),
    (0x3709, 0x12),
#    (0x3808, 0x07),
#    (0x3809, 0x80),
#    (0x380a, 0x04),
#    (0x380b, 0x38),
    (0x3800, 0x01),
    (0x3801, 0x5c),
    (0x3802, 0x01),
    (0x3803, 0xb2),
    (0x3804, 0x08),
    (0x3805, 0xe3),
    (0x3806, 0x05),
    (0x3807, 0xf1),
    (0x3811, 0x04),
    (0x3813, 0x02),
    (0x3630, 0x2e),
    (0x3632, 0xe2),
    (0x3633, 0x23),
    (0x3634, 0x44),
    (0x3636, 0x06),
    (0x3620, 0x64),
    (0x3621, 0xe0),
    (0x3600, 0x37),
    (0x3704, 0xa0),
    (0x3703, 0x5a),
    (0x3715, 0x78),
    (0x3717, 0x01),
    (0x3731, 0x02),
    (0x370b, 0x60),
    (0x3705, 0x1a),
    (0x3f05, 0x02),
    (0x3f06, 0x10),
    (0x3f01, 0x0a),
    (0x3a08, 0x01),
    (0x3a09, 0x4b),
    (0x3a0a, 0x01),
    (0x3a0b, 0x13),
    (0x3a0d, 0x04),
    (0x3a0e, 0x03),
#    (0x3a0f, 0x58),
#    (0x3a10, 0x50),
#    (0x3a1b, 0x58),
#    (0x3a1e, 0x50),
#    (0x3a11, 0x60),
#    (0x3a1f, 0x28),
    (0x4001, 0x02),
    (0x4004, 0x04),
    (0x4000, 0x09),
    (0x4837, 0x19),
    (0x4800, 0x34),
#    (0x3503, 0x03),
    (0x5180, 0x40),

    (0x3503, 0x00               ),  # [1] AGC manual, [0] AEC manual (def.0x00)

    (0x3a11, 0x40				),	# [7:0] vpt_high (def.0xd0)
    (0x3a1f, 0x08				),	# [7:0] vpt_low  (def.0x40)
    (0x3a0f, 0x20				),	# [7:0] wpt  (def.0x78)
    (0x3a10, 0x18				),	# [7:0] bpt  (def.0x68)
    (0x3a1b, 0x20				),	# [7:0] wpt2 (def.0x78)
    (0x3a1e, 0x18				),	# [7:0] bpt2 (def.0x68)
    (0x3a18, 0x00               ),  # [1:0] gain_ceiling[9:8] (def.0x00)
    (0x3a19, 0x80               ),  # [7:0] gain_ceiling[7:0] (def.0x7c)
    (0x5001, 0x01               ),  # [0] awb_en (def.0x01)

    (0x3800, 0x01),
    (0x3801, 0x5c),
    (0x3802, 0x01),
    (0x3803, 0xb2),
    (0x3804, 0x08),
    (0x3805, 0xe3),
    (0x3806, 0x05),
    (0x3807, 0xf1),
    (0x3800, 600        >> 8    ),  # [3:0] x_addr_start[11:8]
    (0x3801, 600        &  0xff ),  # [7:0] x_addr_start[ 7:0]
    (0x3802, 438        >> 8    ),  # [3:0] y_addr_start[11:8]
    (0x3803, 438        &  0xff ),  # [7:0] y_addr_start[ 7:0]
    (0x3804, (2048-1)   >> 8    ),  # [3:0] x_addr_end[11:8]
    (0x3805, (2048-1)   &  0xff ),  # [7:0] x_addr_end[ 7:0]
    (0x3806, (1462-1)   >> 8    ),  # [3:0] y_addr_end[11:8]
    (0x3807, (1462-1)   &  0xff ),  # [7:0] y_addr_end[ 7:0]

    (0x3808, 1280       >> 8    ),  # [3:0] DVP output horizontal width[11:8]
    (0x3809, 1280       &  0xff ),	# [7:0] DVP output horizontal width[ 7:0]
    (0x380a, 960       >> 8    ),	# [3:0] DVP output vertical height[11:8]
    (0x380b, 960       &  0xff ),	# [7:0] DVP output vertical height[ 7:0]
    (0x380e, 1040       >> 8    ),  # [3:0] Total vertical size[9:8]
    (0x380f, 1040       &  0xff ),	# [7:0] Total vertical size[7:0]

#	(0x3820, 0x46               ),  # [2] r_vflip_isp,  [1] r_vflip_snr,  [0] Vertical binning   (def.0x40)
#	(0x3821, 0x00               ),  # [2] r_mirror_isp, [1] r_mirror_snr, [0] Horizontal binning (def.0x00)

    (0x0100, 0x01),

], dtype=dtype )


mode_640x480    = np.array( [

    (0x0100, 0x00),
    (0x0103, 0x01),
    (0x3034, 0x1a),
    (0x3035, 0x21),
    (0x3036, 0x62),
    (0x303c, 0x11),
    (0x3106, 0xf5),
    (0x3827, 0xec),
    (0x370c, 0x03),
    (0x3612, 0x59),
    (0x3618, 0x00),
    (0x5000, 0x06),
    (0x5002, 0x41),
    (0x5003, 0x08),
    (0x5a00, 0x08),
    (0x3000, 0x00),
    (0x3001, 0x00),
    (0x3002, 0x00),
    (0x3016, 0x08),
    (0x3017, 0xe0),
    (0x3018, 0x44),
    (0x301c, 0xf8),
    (0x301d, 0xf0),
#    (0x3a18, 0x00),
#    (0x3a19, 0xf8),
    (0x3c01, 0x80),
    (0x3b07, 0x0c),
#    (0x3800, 0x00),
#    (0x3801, 0x00),
#    (0x3802, 0x00),
#    (0x3803, 0x00),
#    (0x3804, 0x0a),
#    (0x3805, 0x3f),
#    (0x3806, 0x07),
#    (0x3807, 0xa3),
#    (0x3808, 0x05),
#    (0x3809, 0x10),
#    (0x380a, 0x03),
#    (0x380b, 0xcc),
#    (0x380c, 0x07),
#    (0x380d, 0x68),
    (0x3811, 0x0c),
    (0x3813, 0x06),
    (0x3814, 0x31),
    (0x3815, 0x31),
    (0x3630, 0x2e),
    (0x3632, 0xe2),
    (0x3633, 0x23),
    (0x3634, 0x44),
    (0x3636, 0x06),
    (0x3620, 0x64),
    (0x3621, 0xe0),
    (0x3600, 0x37),
    (0x3704, 0xa0),
    (0x3703, 0x5a),
    (0x3715, 0x78),
    (0x3717, 0x01),
    (0x3731, 0x02),
    (0x370b, 0x60),
    (0x3705, 0x1a),
    (0x3f05, 0x02),
    (0x3f06, 0x10),
    (0x3f01, 0x0a),
    (0x3a08, 0x01),
    (0x3a09, 0x28),
    (0x3a0a, 0x00),
    (0x3a0b, 0xf6),
    (0x3a0d, 0x08),
    (0x3a0e, 0x06),
#    (0x3a0f, 0x58),
#    (0x3a10, 0x50),
#    (0x3a1b, 0x58),
#    (0x3a1e, 0x50),
#    (0x3a11, 0x60),
#    (0x3a1f, 0x28),
    (0x4001, 0x02),
    (0x4004, 0x04),
    (0x4000, 0x09),
    (0x4837, 0x16),
    (0x4800, 0x24),
#    (0x3503, 0x03),
    (0x3820, 0x41),
    (0x3821, 0x07),
    (0x350a, 0x00),
    (0x350b, 0x10),
    (0x3500, 0x00),
    (0x3501, 0x1a),
    (0x3502, 0xf0),
    (0x3212, 0xa0),

    (0x3503, 0x00               ),  # [1] AGC manual, [0] AEC manual (def.0x00)
    (0x3a11, 0x40				),	# [7:0] vpt_high (def.0xd0)
    (0x3a1f, 0x08				),	# [7:0] vpt_low  (def.0x40)
    (0x3a0f, 0x20				),	# [7:0] wpt  (def.0x78)
    (0x3a10, 0x18				),	# [7:0] bpt  (def.0x68)
    (0x3a1b, 0x20				),	# [7:0] wpt2 (def.0x78)
    (0x3a1e, 0x18				),	# [7:0] bpt2 (def.0x68)
    (0x3a18, 0x00               ),  # [1:0] gain_ceiling[9:8] (def.0x00)
    (0x3a19, 0x80               ),  # [7:0] gain_ceiling[7:0] (def.0x7c)
    (0x5001, 0x01               ),  # [0] awb_en (def.0x01)

    (0x3800, 512        >> 8    ),  # [3:0] x_addr_start[11:8]
    (0x3801, 512        &  0xff ),  # [7:0] x_addr_start[ 7:0]
    (0x3804, (2048-1)   >> 8    ),  # [3:0] x_addr_end[11:8]
    (0x3805, (2048-1)   &  0xff ),  # [7:0] x_addr_end[ 7:0]
    (0x3802, 448        >> 8    ),  # [3:0] y_addr_start[11:8]
    (0x3803, 448        &  0xff ),  # [7:0] y_addr_start[ 7:0]
    (0x3806, (1440-1)   >> 8    ),  # [3:0] y_addr_end[11:8]
    (0x3807, (1440-1)   &  0xff ),  # [7:0] y_addr_end[ 7:0]
    (0x3808, 640        >> 8    ),  # [3:0] DVP output horizontal width[11:8]
    (0x3809, 640        &  0xff ),	# [7:0] DVP output horizontal width[ 7:0]
    (0x380a, 480        >> 8    ),	# [3:0] DVP output vertical height[11:8]
    (0x380b, 480        &  0xff ),	# [7:0] DVP output vertical height[ 7:0]
    (0x380e, 504        >> 8    ),	# [3:0] Total vertical size[9:8]
    (0x380f, 504        &  0xff ),	# [7:0] Total vertical size[7:0]

#	(0x3820, 0x47               ),  # [2] r_vflip_isp,  [1] r_vflip_snr  (def.0x40)
#	(0x3821, 0x01               ),  # [2] r_mirror_isp, [1] r_mirror_snr (def.0x00)

    (0x0100, 0x01),

], dtype=dtype )


def main(args):
    global MESSAGE_ON
    MESSAGE_ON = args.verbose

    SCL_WIDTH = args.width
    SCL_HEIGHT = args.hight

    if args.res == 1080:
        VIDEO_RES = "1080P"
        print('1080p resolution selected') 
    elif args.res == 960:
        VIDEO_RES = "960P"
        print('960p resolution selected') 
    else:        
        VIDEO_RES = "480P"
        print('480p resolution selected') 

    # Open resources
    fd_reg          = os.open( '/dev/uio0', os.O_RDWR|os.O_SYNC )
    reg_size        = math.ceil( LW_BRIDGE_SIZE/mmap.PAGESIZE )* mmap.PAGESIZE
    reg_mem         = mmap.mmap( fd_reg, reg_size, mmap.MAP_SHARED, mmap.PROT_READ|mmap.PROT_WRITE, offset=0 )

    # Reset camera input path
    reg_mem.seek ( PIO_OFFSET + ALTPIO_DATA * 4 )
    reg_mem.write( (0x0000).to_bytes(4, 'little'))
    sleep( 0.010 )

    # Activation camera input path
    reg_mem.seek ( PIO_OFFSET + ALTPIO_DATA * 4 )
    if      VIDEO_RES == "1080P":
        reg_mem.write( (0x030e).to_bytes(4, 'little') )
    elif    VIDEO_RES == "960P":
        reg_mem.write( (0x032e).to_bytes(4, 'little') )
    elif    VIDEO_RES == "480P":
        reg_mem.write( (0x033e).to_bytes(4, 'little') )

    sleep( 0.100 )

    # Initialze camera input path
    m2p_init()
    cis_init( VIDEO_RES, DATA_MODE )
    sleep( 0.010 )

    reg_mem.seek ( SCL_OFFSET + ALTSCL_WIDTH   * 4 ) 
    reg_mem.write( SCL_WIDTH.to_bytes(4, 'little') )
    reg_mem.seek ( SCL_OFFSET + ALTSCL_HEIGHT  * 4 )
    reg_mem.write( SCL_HEIGHT.to_bytes(4, 'little'))
    reg_mem.seek ( SCL_OFFSET + ALTSCL_CONTROL * 4 )
    reg_mem.write( (0x1).to_bytes(4, 'little') )          # Scaler Start

    # Start Camera input
    cis_start()
    sleep( 0.100 )

    # Enable camera input into FPGA
    reg_mem.seek ( PIO_OFFSET + ALTPIO_DATA * 4 )
    if      VIDEO_RES == "1080P":
        reg_mem.write( (0x030f).to_bytes(4, 'little') )
    elif    VIDEO_RES == "960P":
        reg_mem.write( (0x032f).to_bytes(4, 'little') )
    elif    VIDEO_RES == "480P":
        reg_mem.write( (0x033f).to_bytes(4, 'little') )

    reg_mem.close()
    os.close( fd_reg )


def cis_i2c_write( reg_addr, data ):
    command     = [reg_addr >> 8, reg_addr & 0xff, data]
    write       = i2c_msg.write( I2C_SADR_OV5647, command )

    with SMBus(1) as i2c:
        i2c.i2c_rdwr( write )
        if MESSAGE_ON:
            print( 'Register Write OV5647   : addr=0x%04x data=0x%02x' %(reg_addr, data) )


def cis_i2c_read( reg_addr ):
    command     = [reg_addr >> 8, reg_addr & 0xff]
    write       = i2c_msg.write( I2C_SADR_OV5647, command )
    read        = i2c_msg.read ( I2C_SADR_OV5647, 1 )

    with SMBus(1) as i2c:
        i2c.i2c_rdwr( write, read )
        data        = list( read )
        if MESSAGE_ON:
            print( 'Register Read  OV5647   : addr=0x%04x data=0x%02x' %(reg_addr, data[0]) )

    return data[0]


def m2p_i2c_write( reg_addr, data ):
    command     = [reg_addr >> 8, reg_addr & 0xff, data >> 8, data & 0xff]
    write       = i2c_msg.write( I2C_SADR_TC358748, command )

    with SMBus(1) as i2c:
        i2c.i2c_rdwr( write )
        if MESSAGE_ON:
            print( 'Register Write TC358748 : addr=0x%04x data=0x%04x' %(reg_addr, data) )


def m2p_i2c_read( reg_addr ):
    command     = [reg_addr >> 8, reg_addr & 0xff]
    write       = i2c_msg.write( I2C_SADR_TC358748, command )
    read        = i2c_msg.read ( I2C_SADR_TC358748, 2 )

    with SMBus(1) as i2c:
        i2c.i2c_rdwr( write, read )
        data        = list( read )
        if MESSAGE_ON:
            print( 'Register Read  TC358748 : addr=0x%04x data=0x%02x%02x' %(reg_addr, data[0], data[1]) )


def cis_init( video_res, data_mode ) :
    print( '### OV5647    Initialize start -->' )

    cis_i2c_write( 0x3000, 0x0f )
    cis_i2c_write( 0x3001, 0xff )
    cis_i2c_write( 0x3002, 0xe4 )

    if      video_res   == "1080P":
        for name in mode_1920x1080:
            cis_i2c_write( name['adrs'], name['data'] )
    elif    video_res   == "960P":
        for name in mode_1280x960:
            cis_i2c_write( name['adrs'], name['data'] )
    elif    video_res   == "480P":
        for name in mode_640x480:
            cis_i2c_write( name['adrs'], name['data'] )

    print( '--> OV5647 Initialize end ###' )


def m2p_init():
    print( '### TC358748 Initialize start -->' )

    print( 'TC358748 Software Reset' )
    m2p_i2c_write( 0x0002, 0x0001 )         # SYSctl, S/W Reset
    sleep( 0.0001 )
    m2p_i2c_write( 0x0002, 0x0000 )         # SYSctl, S/W Reset release

    print( 'TC358748 PLL,Clock Setting' )
    m2p_i2c_write( 0x0016, 0x0015 )         # PLL Control Register 0 (PLL_PRD,PLL_FBD)
    m2p_i2c_write( 0x0018, 0x0403 )         # PLL_FRS,PLL_LBWS, PLL oscillation enable
    sleep( 0.001 )
    m2p_i2c_write( 0x0018, 0x0413 )         # PLL_FRS,PLL_LBWS, PLL clock out enable
    m2p_i2c_write( 0x0020, 0x0011 )         # CLK control register: Clock divider setting
    m2p_i2c_write( 0x000C, 0x0101 )         # MCLK duty setting

    print( 'TC358748 Format configuration, timing Setting' )
    m2p_i2c_write( 0x0060, 0x8009 )         # MIPI PHY Time Delay

    m2p_i2c_write( 0x0006, 0x0010 )         # FIFO control
    m2p_i2c_write( 0x0008, 0x0010 )         # Data format control
    m2p_i2c_write( 0x0004, 0x004d )         # Configuration control

    print( '--> TC358748 Initialize end ###' )


def cis_start() :
    print( '### OV5647 Streaming -->' )

    cis_i2c_write( 0x4800, 0x04 )
    cis_i2c_write( 0x4202, 0x00 )
    cis_i2c_write( 0x300D, 0x00 )
    sleep( 0.001 )

#    cis_i2c_write( 0x503D, 0x80 )
    

def cis_stop() :
    print( '--> OV5647 Standby ###' )
    
    cis_i2c_write( 0x4800, 0x25 )
    cis_i2c_write( 0x4202, 0x0f )
    cis_i2c_write( 0x300D, 0x01 )
    sleep( 0.001 )

    cis_i2c_write( 0x3000, 0x00 )
    cis_i2c_write( 0x3001, 0x00 )
    cis_i2c_write( 0x3002, 0x00 )

def get_option():
    parser = argparse.ArgumentParser(description='This toos sets the KEIm-CVSoC built-in camera module')
    parser.add_argument('-r', '--res', type=int, default=1080,
                        help='Definition of resolution (480 or 960 or 1080)')
    parser.add_argument('-hw', '--width', type=int, default=640,
                        help='Set the horizontal size of the scaler')
    parser.add_argument('-vh', '--hight', type=int, default=480,
                        help='Set the vertical size of the scaler')
    parser.add_argument('-v', '--verbose', type=bool, default=False,
                        help='View executed commands')
    return parser.parse_args()

if __name__ == '__main__':
    args = get_option()
    main(args)

