//==============================================================================
//	File Name	: BURST_BUFFER.v
//	Version		: 1.1
//	Date		: 2021/06/07
//	Model		: BURST_BUFFER
//
//==============================================================================

module BURST_BUFFER (
	input	wire				RST_N		,
	input	wire				AVCLK		,
	// -- From AV_SINK -- //
	input	wire				IV_VLD		,
	input	wire	[23:0]		IV_DT		,
	input	wire				IV_LAST		,
	output	wire				OV_RDY		,
	// -- To AM_MASER -- //
	output	wire				OWREQ		,
	output	wire				OFRM_LAST	,
//	output	wire	[3:0]		OWNUM		,
	input	wire				IWACK		,
	input	wire				IBUF_RE		,
	input	wire	[3:0]		IBUF_RADDR	,
	output	wire	[255:0]		OBUF_RDT
);

//==============================================================================
// Parameter
//==============================================================================

//==============================================================================
// Register & Wire
//==============================================================================
reg					v_vld			;
reg		[23:0]		v_dt			;
reg		[23:0]		v_dt_s1			;
reg					v_last			;

reg		[1:0]		map_pat			;
wire				v_vld_p1_w		;
reg					v_vld_p1		;

wire				v_last_s1_w		;
reg					v_last_s1		;
reg					wen				;
reg		[6:0]		waddr			;
reg		[1:0]		wbnk			;
wire	[31:0]		wdata_w			;
reg		[31:0]		wdata			;

wire				wlast_w			;
wire				wlast0_w		;
wire				wlast1_w		;
wire				wlast2_w		;
wire				wlast3_w		;
reg					wlast0			;
reg					wlast1			;
reg					wlast2			;
reg					wlast3			;
reg					wbnk_inc		;
reg		[1:0]		bnk_cnt			;
wire				v_rdy_w			;
reg					v_rdy			;
reg					wr_ack			;
reg					wr_req			;
//reg		[3:0]		wr_num			;
reg					frm_last		;
reg		[1:0]		rbnk			;
//wire	[3:0]		rsize_sel		;
wire				rlast_sel		;
wire	[8:0]		buf_waddr		;
wire	[5:0]		buf_raddr		;


//==============================================================================
// Description of Circuit
//==============================================================================

// Input FF
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
		v_vld		<= 1'b0;
		v_dt		<= 24'd0;
		v_dt_s1		<= 24'd0;
		v_last		<= 1'b0;
	end
	else begin
		v_vld		<= IV_VLD;
		v_dt		<= IV_DT;
		v_dt_s1		<= v_dt;
		v_last		<= IV_LAST;
	end
end

//----------------------------------------------------------
// 24bit->32bit conversion
//----------------------------------------------------------
// Mapping Pettern
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)								map_pat		<= 2'd0;
	else if (v_vld)
		if (v_last & map_pat==2'd3)			map_pat		<= 2'd0;
		else								map_pat		<= map_pat + 2'd1;
	else if (v_vld_p1)						map_pat		<= 2'd0;			// last if not divisible by 4
	else									map_pat		<= map_pat;
end

// When v_last is not set with map_pat=3 (= input is not divisible by 4), i.e. 24bit to 32bit conversion,
// a fraction (not all 32bit is filled) occurs in the last data.
// To write the fraction, increase v_vld by one cycle, and delay v_last by one cycle.

assign v_vld_p1_w = (v_vld & v_last & (map_pat!=2'd3));

always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)						v_vld_p1	<= 1'b0;
	else							v_vld_p1	<= v_vld_p1_w;
end



//----------------------------------------------------------
// Buffer Control
//----------------------------------------------------------
// last of frame after 24bit32bit conversion
assign v_last_s1_w = ((v_vld & v_last & map_pat==2'd3) | v_vld_p1);

always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)			v_last_s1		<= 1'b0;
	else				v_last_s1		<= v_last_s1_w;
end


// Memory I/F

// Buffer Write
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) 						wen		<= 1'b0;
	else if (v_vld & (map_pat!='d0))	wen		<= 1'b1;
	else if (v_vld_p1)					wen		<= 1'b1;	// when not divisible by 4
	else								wen		<= 1'b0;
end

// Buffer Address
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) 						waddr	<= 7'd0;
	else if (wen)
		if (waddr==7'h7F | v_last_s1)	waddr	<= 7'd0;	// last of bank
		else							waddr	<= waddr + 1;
	else								waddr	<= waddr;
end


// Write Data
assign wdata_w = (map_pat==2'd0)? 32'd0						:
				 (map_pat==2'd1)? {v_dt[7:0], v_dt_s1[23:0]}	:
				 (map_pat==2'd2)? {v_dt[15:0], v_dt_s1[23:8]}:
				 {v_dt[23:0], v_dt_s1[23:16]};

always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) 						wdata	<= 32'd0;
	else if (v_vld | v_vld_p1)			wdata	<= wdata_w;
	else								wdata	<= wdata;
end

// Write Bank
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) 									wbnk	<= 2'd0;
	else if (wen & (waddr==7'h7F | v_last_s1))		wbnk	<= wbnk + 1;	// last of bank
	else											wbnk	<= wbnk;
end


// Keep last of frame in each bank.
assign wlast_w = (wen & v_last_s1)? 1'b1 : 1'b0;

assign wlast0_w	= (wbnk==2'd0)? wlast_w : wlast0;
assign wlast1_w	= (wbnk==2'd1)? wlast_w : wlast1;
assign wlast2_w	= (wbnk==2'd2)? wlast_w : wlast2;
assign wlast3_w	= (wbnk==2'd3)? wlast_w : wlast3;

always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
			wlast0	<= 1'b0;
			wlast1	<= 1'b0;
			wlast2	<= 1'b0;
			wlast3	<= 1'b0;
	end
	else begin
		if (wen & (waddr==7'h7F | v_last_s1)) begin
			wlast0	<= wlast0_w;
			wlast1	<= wlast1_w;
			wlast2	<= wlast2_w;
			wlast3	<= wlast3_w;
		end else begin
			wlast0	<= wlast0;
			wlast1	<= wlast1;
			wlast2	<= wlast2;
			wlast3	<= wlast3;
		end
	end
end




// Bank Strage Count Management
// Write
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)										wbnk_inc	<= 1'b0;
	else if (wen & (waddr==7'h7F | v_last_s1))		wbnk_inc	<= 1'b1;
	else											wbnk_inc	<= 1'b0;
end

// bnk_cnt=0(empty)-4(full)
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)				bnk_cnt	<= 2'd0;
	else if (wbnk_inc)
			if (wr_ack)			bnk_cnt	<= bnk_cnt;				// write and read
			else				bnk_cnt	<= bnk_cnt + 2'd1;		// only write
	else if (wr_ack)			bnk_cnt	<= bnk_cnt - 2'd1;		// only read
	else						bnk_cnt	<= bnk_cnt;
end

assign v_rdy_w = (bnk_cnt==2'd3)? 1'b0 : 1'b1;	// Clear to receive with 3 sides stored.


always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)				v_rdy	<= 1'b0;
	else					v_rdy	<= v_rdy_w;
end

assign OV_RDY = v_rdy;

// DDR Write Ack
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N)			wr_ack	<= 1'b0;
	else				wr_ack	<= IWACK;
end


// DDR Write Request
always @(posedge AVCLK or negedge RST_N) begin
	if (~RST_N) begin
			wr_req		<= 1'b0;
//			wr_num		<= 4'd0;
			frm_last	<= 1'b0;
			rbnk		<= 2'd0;
	end
	else if (wr_ack) begin
			wr_req		<= 1'b0;
//			wr_num		<= wr_num;
			frm_last	<= 1'b0;
			rbnk		<= rbnk + 2'd1;
	end
	else if (~wr_req & ~wr_ack) begin	// next command after clear REQ and ACK
		if (bnk_cnt!=2'd0) begin
				wr_req		<= 1'b1;
//				wr_num		<= rsize_sel;
				frm_last	<= rlast_sel;
				rbnk		<= rbnk;
		end
		else begin
				wr_req		<= 1'b0;
//				wr_num		<= wr_num;
				frm_last	<= frm_last;
				rbnk		<= rbnk;
		end
	end
	else begin
			wr_req		<= wr_req;
//			wr_num		<= wr_num;
			frm_last	<= frm_last;
			rbnk		<= rbnk;
	end
end

assign OWREQ     = wr_req;
//assign OWNUM     = wr_num;
assign OFRM_LAST = frm_last;

assign rlast_sel = (rbnk==2'd0)? wlast0 :
				   (rbnk==2'd1)? wlast1 :
				   (rbnk==2'd2)? wlast2 :
				   wlast3;


//----------------------------------------------------------
// Buffer
//----------------------------------------------------------
assign buf_waddr = {wbnk, waddr};
assign buf_raddr = {rbnk, IBUF_RADDR};

altsyncram bufram (
	.address_a  (buf_waddr),// input	[8:0]  wraddress;
	.address_b  (buf_raddr),// input	[5:0]  rdaddress;
	.clock0     (AVCLK),    // input	  clock;
	.data_a     (wdata),    // input	[31:0]  data;
	.rden_b     (IBUF_RE),  // input	  rden;
	.wren_a     (wen),      // input	  wren;
	.q_b        (OBUF_RDT), // output	[255:0]  q;
	.aclr0      (1'b0),
	.aclr1      (1'b0),
	.addressstall_a (1'b0),
	.addressstall_b (1'b0),
	.byteena_a  (1'b1),
	.byteena_b  (1'b1),
	.clock1     (1'b1),
	.clocken0   (1'b1),
	.clocken1   (1'b1),
	.clocken2   (1'b1),
	.clocken3   (1'b1),
	.data_b     ({256{1'b1}}),
	.eccstatus  (),
	.q_a        (),
	.rden_a     (1'b1),
	.wren_b     (1'b0)
    );

	defparam
		bufram.address_aclr_b = "NONE",
		bufram.address_reg_b = "CLOCK0",
		bufram.clock_enable_input_a = "BYPASS",
		bufram.clock_enable_input_b = "BYPASS",
		bufram.clock_enable_output_b = "BYPASS",
		bufram.intended_device_family = "Cyclone V",
		bufram.lpm_type = "altsyncram",
		bufram.numwords_a = 512,
		bufram.numwords_b = 64,
		bufram.operation_mode = "DUAL_PORT",
		bufram.outdata_aclr_b = "NONE",
		bufram.outdata_reg_b = "CLOCK0",
		bufram.power_up_uninitialized = "FALSE",
		bufram.rdcontrol_reg_b = "CLOCK0",
		bufram.read_during_write_mode_mixed_ports = "DONT_CARE",
		bufram.widthad_a = 9,
		bufram.widthad_b = 6,
		bufram.width_a = 32,
		bufram.width_b = 256,
		bufram.width_byteena_a = 1;

endmodule
